//
// Created by philipp on 05.02.18.
//

#ifndef PC2CAR_COMMANDRECEIVER_H
#define PC2CAR_COMMANDRECEIVER_H

#include "PC2Car.h"
#include "NetworkingLib/DatagramReceiver.h"
#include "NetworkingLib/TimedValue.h"
#include "PlatoonProtocolLib/Protocol.h"
#include "json.hpp"

namespace pc2car
{

class CommandReceiver : public std::enable_shared_from_this<CommandReceiver>
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<CommandReceiver>;
    using DatagramReceiver = networking::message::DatagramReceiver<std::string>;
    using OnCommandReceivedCallback = std::function<void(CommandCode)>;

    static Ptr create(
        networking::Networking & net,
        bool defaultLoggingEnabled = false,
        bool defaultPlatoonEnabled = false,
        bool defaultRCModeEnabled = false,
        float defaultPlatoonSpeed = 0.0f,
        float defaultInnerPlatoonDistance = 0.0f,
        float defaultSpeed = 0.0f);

    CommandReceiver(PrivateTag,
                    networking::Networking & net,
                    bool defaultLoggingEnabled,
                    bool defaultPlatoonEnabled,
                    bool defaultRCModeEnabled,
                    float defaultPlatoonSpeed,
                    float defaultInnerPlatoonDistance,
                    float defaultSpeed);

    void receiveCommands(const OnCommandReceivedCallback & handler);

    void stop();

    networking::time::TimedValue<bool> isLoggingEnabled()
    { return loggingEnabled.getNonAtomicCopy(); }

    networking::time::TimedValue<bool> isPlatoonEnabled()
    { return platoonEnabled.getNonAtomicCopy(); }

    networking::time::TimedValue<bool> isRcModeEnabled()
    { return rcModeEnabled.getNonAtomicCopy(); }

    networking::time::TimedValue<platoonProtocol::PlatoonSpeed> getPlatoonSpeed()
    { return platoonSpeed.getNonAtomicCopy(); }

    networking::time::TimedValue<platoonProtocol::InnerPlatoonDistance> getInnerPlatoonDistance()
    { return innerPlatoonDistance.getNonAtomicCopy(); }

    networking::time::TimedValue<float> getSpeed()
    { return speed.getNonAtomicCopy(); }

private:
    DatagramReceiver::Ptr receiver;
    OnCommandReceivedCallback handler;

    networking::time::TimedAtomicValue<bool> loggingEnabled{false};
    networking::time::TimedAtomicValue<bool> platoonEnabled{false};
    networking::time::TimedAtomicValue<bool> rcModeEnabled{false};
    networking::time::TimedAtomicValue<platoonProtocol::PlatoonSpeed> platoonSpeed{false};
    networking::time::TimedAtomicValue<platoonProtocol::InnerPlatoonDistance> innerPlatoonDistance{false};
    networking::time::TimedAtomicValue<float> speed{false};

    void receiveCommand();

    void setValue(CommandCode code, nlohmann::json & j);
};

}

#endif //PC2CAR_COMMANDRECEIVER_H
