#include "../include/MessageModel.h"

MessageModel::MessageModel(QObject * parent)
    : QAbstractListModel(parent)
      , messageList(nullptr)
{
}

int MessageModel::rowCount(const QModelIndex & parent) const
{
    // For list models only the root node (an invalid parent) should return the list's size. For all
    // other (valid) parents, rowCount() should return 0 so that it does not become a tree model.
    if (parent.isValid() || !messageList)
        return 0;

    return messageList->getItems().size();
}

QVariant MessageModel::data(const QModelIndex & index, int role) const
{
    if (!index.isValid() || !messageList)
        return QVariant{};

    auto message = messageList->getItems().at(index.row());
    switch (role)
    {
        case ModuleRole:
            return QVariant{QString::fromStdString(message.module)};

        case KeyRole:
            return QVariant{QString::fromStdString(message.key)};

        case ValueRole:
            return QVariant{QString::fromStdString(message.value)};
    }

    return QVariant{};
}

QHash<int, QByteArray> MessageModel::roleNames() const
{
    QHash<int, QByteArray> names;
    names[ModuleRole] = "module";
    names[KeyRole] = "key";
    names[ValueRole] = "value";
    return names;
}

MessageList * MessageModel::getMessageList() const noexcept
{
    return messageList;
}

void MessageModel::setMessageList(MessageList * newMessageList)
{
    beginResetModel();

    if (messageList)
        messageList->disconnect(this);

    messageList = newMessageList;

    if (messageList)
    {
        connect(messageList, &MessageList::preMessageAppended, this, [this]
        {
            int index = messageList->getItems().size();
            beginInsertRows(QModelIndex{}, index, index);
        });

        connect(messageList, &MessageList::postMessageAppended, this, [this]
        {
            endInsertRows();
        });
    }

    endResetModel();
}
